/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef ARM4DBBACKEND_H	/* Prevent multiple inclusion */
#define ARM4DBBACKEND_H 1

#include "arm4_db.h"

class Arm4dbBackEnd {
public:
	/* Constructors */

	/* Destructors */
	virtual ~Arm4dbBackEnd () {}

	/* Accessors */

	/* Mutators */

	/* Operations */
	virtual void initializeLibrary (void) = 0;
	virtual void initializeThread (void) = 0;

	virtual arm_error_t openDatabases (void) = 0;
	virtual arm_error_t openDatabasesReadOnly (void) = 0;
	virtual arm_error_t closeDatabasesNoLock (void) = 0;

	virtual arm_error_t initializeSequences (arm4_shm_header_t *header_ptr) = 0;

	virtual arm_error_t processApplicationInstances (const arm4_shm_application_instance_t *app_instance_ptr) = 0;
	virtual arm_error_t processApplicationGroup (const arm4_shm_property_t *group_ptr) = 0;
	virtual arm_error_t processApplicationInstanceId (const arm4_shm_property_t *property_ptr) = 0;
	virtual arm_error_t processApplicationContext (const arm4_shm_context_property_t *property_ptr) = 0;
	virtual arm_error_t processApplicationAddress (const arm4_shm_address_t *address_ptr) = 0;
	virtual arm_error_t processTransactionInstances (const arm4_shm_transaction_instance_t *tran_instance_ptr) = 0;
	virtual arm_error_t processTransactionContext (const arm4_shm_context_property_t *property_ptr) = 0;
	virtual arm_error_t processTransactionUri (const arm4_shm_uri_t *uri_ptr) = 0;
	virtual arm_error_t processTransactionMetricValue (const arm4_shm_metric_value_t *metric_ptr) = 0;
	virtual arm_error_t processTransactionUser (const arm4_shm_user_t *user_ptr) = 0;
	virtual arm_error_t processTransactionDiag (const arm4_shm_diag_t *diag_ptr) = 0;
	virtual arm_error_t processCorrelators (const arm4_shm_correlator_t *correlator_ptr) = 0;
	virtual arm_error_t processSequence (const arm4_shm_db_sequence_t *sequence_ptr) = 0;
	virtual arm_error_t processRegisterApplication (const arm4_shm_register_application_t *application_ptr,
		arm_id_t *appid_ptr,
		arm_boolean_t *is_new_ptr) = 0;
	virtual arm_error_t processRegisterApplicationIdentity (const arm4_shm_register_application_identity_t *identity_ptr) = 0;
	virtual arm_error_t processRegisterApplicationContext (const arm4_shm_register_application_context_t *context_ptr) = 0;
	virtual arm_error_t processRegisterTransaction (const arm4_shm_register_transaction_t *transaction_ptr,
		arm_id_t *tranid_ptr,
		arm_boolean_t *is_new_ptr) = 0;
	virtual arm_error_t processRegisterTransactionIdentity (const arm4_shm_register_application_identity_t *identity_ptr) = 0;
	virtual arm_error_t processRegisterTransactionContext (const arm4_shm_register_application_context_t *context_ptr) = 0;
	virtual arm_error_t processRegisterTransactionUri (const arm4_shm_register_transaction_uri_t *uri_ptr) = 0;
	virtual arm_error_t processRegisterTransactionMetricBinding (const arm4_shm_register_transaction_metric_t *metric_ptr) = 0;
	virtual arm_error_t processRegisterMetric (arm4_shm_register_metric_t *metric_ptr, arm_id_t *metricid_ptr) = 0;
	virtual arm_error_t processTraceStats (arm_time_t sample_time, arm4db_transaction_trace_stats_t *stats_ptr, int transaction_count) = 0;

	virtual void saveApplicationDefinition (const arm_id_t &appl_id, const application_record_t &application) = 0;
	virtual void saveApplicationIdentity (const arm_id_t &appl_id, const property_record_t &property) = 0;
	virtual void saveApplicationContext (const arm_id_t &appl_id, const indexed_property_name_record_t &property) = 0;
	
	virtual void saveMetricDefinition (const arm_id_t &metric_id, const metric_record_t &metric) = 0;
	
	virtual void saveTransactionDefinition (const arm_id_t &tran_id, const transaction_record_t &transaction) = 0;
	virtual void saveTransactionIdentity (const arm_id_t &tran_id, const property_record_t &property) = 0;
	virtual void saveTransactionContext (const arm_id_t &tran_id, const indexed_property_name_record_t &property) = 0;
	virtual void saveTransactionUri (const arm_id_t &tran_id, const char *uri_ptr) = 0;
	virtual void saveTransactionMetric (const arm_id_t &tran_id, const arm_metric_binding_t &binding) = 0;
	
	virtual void saveApplicationInstance (const arm_app_start_handle_t app_handle, const application_instance_record_t &application) = 0;
	virtual void saveAppInstanceContextValue (const arm_app_start_handle_t app_handle, const indexed_property_value_record_t &property) = 0;
	virtual void saveAppInstanceGroup (const arm_app_start_handle_t app_handle, const char *group_ptr) = 0;
	virtual void saveAppInstanceName (const arm_app_start_handle_t app_handle, const char *name_ptr) = 0;
	virtual void saveAppInstanceAddress (const arm_app_start_handle_t app_handle, const application_address_record_t &address_value) = 0;

	virtual void saveTransactionInstance (const arm_tran_start_handle_t tran_handle, const transaction_instance_record_t &transaction) = 0;
	virtual void saveTranInstanceContextValue (const arm_tran_start_handle_t tran_handle, const indexed_property_value_record_t &property) = 0;
	virtual void saveTranInstanceMetricValue (const arm_tran_start_handle_t tran_handle, const metric_value_record_t &metric) = 0;
	virtual void saveTranInstanceUri (const arm_tran_start_handle_t tran_handle, const char *uri_ptr) = 0;
	virtual void saveTranInstanceUser (const arm_tran_start_handle_t tran_handle, const user_record_t &user) = 0;
	virtual void saveTranInstanceDiag (const arm_tran_start_handle_t tran_handle, const char *diag_ptr) = 0;
	virtual void saveTranCorrelator (const arm_tran_start_handle_t tran_handle, const arm_correlator_t *correlator_ptr) = 0;
	virtual void saveTranParentCorrelator (const arm_tran_start_handle_t tran_handle, const arm_correlator_t *correlator_ptr) = 0;

	virtual void saveTraceStats (const arm_time_t sample_time, const arm4db_transaction_trace_stats_t &trace_stats) = 0;

	/* Checkpoint the database transactions */
	virtual void checkpoint (void) = 0;
	
	/* Get transaction stats */
	virtual void *getStats (void) = 0;

	/* Debugging routines */
	virtual void dump (void) const = 0;
	
	/* Database backup and rollover */
	virtual void backup (void) = 0;

    /* Reporting functions */
    virtual void printSummaries (void) const = 0;
    virtual void printTransactions (void) const = 0;
    
    virtual std::list<arm_id_t> *getApplicationList (void) = 0;
    virtual std::list<arm_id_t> *getTransactionList (void) = 0;
    virtual std::list<arm_id_t> *getTransactionList (const arm_id_t &app_id) = 0;
	virtual std::list<arm_tran_start_handle_t> *getTransactionInstanceList (const arm_id_t &tran_id) = 0;
	virtual int getApplicationCount (void) = 0;
	virtual int getTransactionCount (void) = 0;
	virtual int getTransactionCount (const arm_id_t &app_id) = 0;
    
    virtual arm_error_t getApplicationInformation (const arm_id_t &app_id, arm4db_application_summary_t *summary_ptr) = 0;
    virtual arm_error_t getTransactionInformation (const arm_id_t &tran_id, arm4db_transaction_summary_t *summary_ptr) = 0;
	virtual const arm_correlator_t *getCorrelator (arm_tran_start_handle_t tran_handle, arm_correlator_t *correlator_ptr) = 0;
	virtual const arm_correlator_t *getParentCorrelator (arm_tran_start_handle_t tran_handle, arm_correlator_t *correlator_ptr) = 0;
    
    virtual void setTransactionTraceInformation (const arm_id_t &tran_id, const arm4db_transaction_summary_t &summary) = 0;
    
    virtual std::ostream& exportApplicationDefinitions (std::ostream &os) const = 0;
    virtual std::ostream& exportTransactionDefinitions (std::ostream &os) const = 0;
    virtual std::ostream& exportMetricDefinitions (std::ostream &os) const = 0;
    virtual std::ostream& exportApplications (std::ostream &os) const = 0;
    virtual std::ostream& exportTransactions (std::ostream &os) const = 0;
    virtual std::ostream& exportTraceStats (std::ostream &os) const = 0;

    std::ostream& exportDefinitions (std::ostream &os) const;
    std::ostream& exportAll (std::ostream &os) const;
    void import (const char *filename_ptr);
	void importFromMemory (const std::string &xml);

	/* Operators */

protected:
	Arm4dbBackEnd () {}

private:
	/* Mark these as private to prevent their use */
	Arm4dbBackEnd (const Arm4dbBackEnd &);
	Arm4dbBackEnd &operator= (const Arm4dbBackEnd &);
};

#endif /* ARM4DBBACKEND_H */
