/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef ARM4DBCOMMON_H	/* Prevent multiple inclusion */
#define ARM4DBCOMMON_H 1

#include <cstring>
#include "Arm4db.h"

#define DB_ASSERT(assert) {Arm4dbCommon::db_assert(assert, __FILE__, __LINE__);}

class Arm4dbCommon {
public:
	/* Constructors */

	/* Destructors */
	virtual ~Arm4dbCommon ();

	/* Accessors */
	static const char *getSequenceName (int index) { return m_sequence_name_table [index]; }

	/* Mutators */

	/* Operations */
	static void db_assert (int assert, const char *file, int line);

	static const char *tempPath (int instance, const char *filename_ptr);
	static arm_error_t createPath (const char *path_ptr);

	static void getUuid (arm_id_t *uuid_ptr);

	/* Operators */
	static void printId (const arm_id_t *id_ptr) {	printId (std::cout, id_ptr); }
	static std::ostream& printId (std::ostream &os, const arm_id_t *id_ptr);
	static std::ostream& printId (std::ostream &os, const arm_id_t &id) { return printId (os, &id); }
	static void printDigest (const char *digest_ptr) {	printDigest (std::cout, digest_ptr); }
	static std::ostream& printDigest (std::ostream &os, const char *digest_ptr);
	static std::ostream& printDigest (std::ostream &os, const char &digest) { return printDigest (os, &digest); }
	static std::ostream& printMetricValue (std::ostream &os, const arm_metric_t *metric_ptr, const std::string &string_value) { return printMetricValue (os, *metric_ptr, string_value); }
	static std::ostream& printMetricValue (std::ostream &os, const arm_metric_t &metric, const std::string &string_value);
	static std::ostream& printCorrelator (std::ostream &os, const arm_correlator_t *correlator_ptr);
	static std::ostream& printCorrelator (std::ostream &os, const arm_correlator_t &correlator) { return printCorrelator(os, &correlator); }
	static void printAddress (const application_address_record_t *address_ptr) { printAddress (std::cout, address_ptr); }
	static std::ostream& printIpV4Address (std::ostream &os, const arm_uint8_t *address_ptr);
	static std::ostream& printIpV4PortAddress (std::ostream &os, const arm_uint8_t *address_ptr);
	static std::ostream& printAddress (std::ostream &os, const arm_uint8_t *address_ptr, int length);
	static std::ostream& printCharacterAddress (std::ostream &os, const arm_uint8_t *address_ptr, int length);
	static std::ostream& printAddress (std::ostream &os, const application_address_record_t *address_ptr);
	static std::ostream& printAddress (std::ostream &os, const application_address_record_t &address) { return printAddress(os, &address); }
	static std::ostream& escapeForXml (std::ostream &os, const std::string &string_value);
	static std::ostream& escapeForXml (std::ostream &os, const char *string_ptr, int max_length);
	static std::ostream& escapeForXml (std::ostream &os, const unsigned char *string_ptr, int max_length) { return escapeForXml (os, (char *) string_ptr, max_length); }

	static int correlatorSize (const arm_correlator_t *correlator_ptr);
	static int correlatorSize (const arm_correlator_t &correlator) { return correlatorSize (&correlator); }

#if defined(HAVE_STRNLEN)
	static size_t stringLength (const char *s, size_t maxlen) { return ::strnlen (s, maxlen); }
#else
	static size_t stringLength (const char *s, size_t maxlen)
	{
		size_t i = 0;
		while (*s && maxlen--)
		{
			i++;
			s++;
		}
		return i;
	}
#endif

private:
	/* Mark these as private to prevent their use */
	Arm4dbCommon ();
	Arm4dbCommon (const Arm4dbCommon &);
	Arm4dbCommon &operator= (const Arm4dbCommon &);

	static const char *createRootTempPath (const char *filename_ptr);
	static const char *createUserTempPath (int instance, const char *filename_ptr);


	static char m_sequence_name_table [arm4_shm_sequence_max][64];
};

#endif /* ARM4DBCOMMON_H */
