/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef ARM4DBDAEMONSHAREDMEMORY_H
#define ARM4DBDAEMONSHAREDMEMORY_H 1

#include "arm4_shm.h"
#include "Arm4dbSharedMemory.h"

class Arm4dbDaemonSharedMemory :
	public Arm4dbSharedMemory
{

public:
	// Constructors
	Arm4dbDaemonSharedMemory ();

	// Destructors
	virtual ~Arm4dbDaemonSharedMemory ();

	// Accessors

	// Mutators

	// Operations
	static void *shmCreate (arm_boolean_t start_as_null);
	static void shmDestroy (void);

	static void setUmask (int umask);
	static pid_t setDaemon (arm_boolean_t force);
	static pid_t getActiveDaemon (void);
	static pid_t clearDaemon (void);

	// Application instance queue
	static int getAppInstances (arm4_shm_application_instance_t *app_instance_ptr);
	static int getAppGroup (arm4_shm_property_t *app_group_ptr);
	static int getAppInstanceId (arm4_shm_property_t *property_ptr);
	static int getAppContext (arm4_shm_context_property_t *property_ptr);
	static int getAppAddress (arm4_shm_address_t *address_ptr);

	// Transaction instance queue
	static int getTranInstances (arm4_shm_transaction_instance_t *tran_instance_ptr);
	static int getTranContext (arm4_shm_context_property_t *property_ptr);
	static int getTranUri (arm4_shm_uri_t *uri_ptr);
	static int getTranMetricValues (arm4_shm_metric_value_t *metric_ptr);
	static int getTranUser (arm4_shm_user_t *user_ptr);
	static int getTranDiag (arm4_shm_diag_t *diag_ptr);

	// Correlator queue
	static int getCorellators (arm4_shm_correlator_t *correlator_ptr);

	// Sequence queue
	static int getSequence (arm4_shm_db_sequence_t *sequence_ptr);

	// Application registration
	static int getRegisterApplication (arm4_shm_register_application_t *message_ptr);
	static void registerApplicationReturn (int application_index, arm_id_t appid, arm_boolean_t is_new);
	static int getRegisterApplicationIdentity (arm4_shm_register_application_identity_t *message_ptr);
	static int getRegisterApplicationContext (arm4_shm_register_application_context_t *message_ptr);

	// Transaction registration
	static int getRegisterTransaction (arm4_shm_register_transaction_t *message_ptr);
	static void registerTransactionReturn (int transaction_index, arm_id_t tran_id, arm_boolean_t is_new);
	static int getRegisterTransactionIdentity (arm4_shm_register_application_identity_t *message_ptr);
	static int getRegisterTransactionContext (arm4_shm_register_application_context_t *message_ptr);
	static int getRegisterTransactionUri (arm4_shm_register_transaction_uri_t *message_ptr);
	static int getRegisterTransactionMetricBinding (arm4_shm_register_transaction_metric_t *message_ptr);

	// Metric registration
	static int getRegisterMetric (arm4_shm_register_metric_t *message_ptr);
	static void registerMetricReturn (int metric_index, arm_id_t metric_id);

	static void getStats (arm4_shm_stats_t *stats_ptr);
	static void sampleFlush (void);

	// Utility commands
	static void utilityCommand (long command);
	static int getUtilityCommand (arm4_shm_utility_t *message_ptr);

	static int waitingMessageCount (void);

	// Operators

private:

	static int getMessage (void *buffer_ptr, int buffer_size, long type);

	static void create_semaphores (void);
	static void destroy_semaphores (void);
};

#endif /* ARM4DBDAEMONSHAREDMEMORY_H */
