/**********************************************************************
 * Copyright (c) 2005-2009 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef ARM4DBEXCEPTIONS_H	/* Prevent multiple inclusion */
#define ARM4DBEXCEPTIONS_H 1

#include <string>

class Arm4dbException
{
public:
	Arm4dbException (const std::string &reason);
    Arm4dbException (const char *format, ...);
	Arm4dbException (const Arm4dbException &e);

	virtual ~Arm4dbException ();

	Arm4dbException &operator= (const Arm4dbException &e);

	const std::string &getReason (void) const { return m_reason; }
	const char *getReasonC (void) const { return m_reason.c_str (); }

protected:
	Arm4dbException ();

	std::string	m_reason;
};

class Arm4dbSharedMemoryException :
	public Arm4dbException
{
public:
	Arm4dbSharedMemoryException () :
		Arm4dbException ("Shared memory")
	{}

	Arm4dbSharedMemoryException (const char *error) :
		Arm4dbException (error)
	{}

	Arm4dbSharedMemoryException (const std::string &error) :
		Arm4dbException (error)
	{}

	Arm4dbSharedMemoryException (const Arm4dbException &error) :
		Arm4dbException (error)
	{}

	~Arm4dbSharedMemoryException ()
	{}
};

class Arm4dbTestDaemonException :
	public Arm4dbException
{
public:
	Arm4dbTestDaemonException () :
		Arm4dbException ("Test Daemon")
	{}

	Arm4dbTestDaemonException (const char *format, ...);

	Arm4dbTestDaemonException (const std::string &error) :
		Arm4dbException (error)
	{}

	Arm4dbTestDaemonException (const Arm4dbException &error) :
		Arm4dbException (error)
	{}

	~Arm4dbTestDaemonException ()
	{}
};

class Arm4dbLockException :
	public Arm4dbException
{
public:
	Arm4dbLockException () :
		Arm4dbException ("Lock")
	{}

	Arm4dbLockException (const char *error) :
		Arm4dbException (error)
	{}

	Arm4dbLockException (const std::string &error) :
		Arm4dbException (error)
	{}

	Arm4dbLockException (const Arm4dbException &error) :
		Arm4dbException (error)
	{}

	~Arm4dbLockException ()
	{}
};

class Arm4dbUninitializedLockException :
	public Arm4dbException
{
public:
	Arm4dbUninitializedLockException () :
		Arm4dbException ("Uninitialized lock")
	{}

	Arm4dbUninitializedLockException (const char *error) :
		Arm4dbException (error)
	{}

	Arm4dbUninitializedLockException (const std::string &error) :
		Arm4dbException (error)
	{}

	Arm4dbUninitializedLockException (const Arm4dbException &error) :
		Arm4dbException (error)
	{}

	~Arm4dbUninitializedLockException ()
	{}
};

#endif /* ARM4DBEXCEPTIONS_H */
