/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef ARM4DBSHAREDMEMORY_H
#define ARM4DBSHAREDMEMORY_H 1

#include "arm4_shm.h"

typedef struct arm4_shm_stats {
	unsigned long	mq_n_messages;
	unsigned long	mq_n_bytes;
	unsigned long	mq_max_bytes;
} arm4_shm_stats_t;

class Arm4dbSharedMemory {

public:
	// Constructors
	Arm4dbSharedMemory ();

	// Destructors
	virtual ~Arm4dbSharedMemory ();

	// Accessors

	// Mutators

	// Operations
	static arm4_shm_header_t *shmAttach (void);

	static arm_boolean_t setNullCollector (arm_boolean_t set_as_null);
	static arm_boolean_t getNullCollector (void);

	// Utility commands
	static void setDebugLevel (int level);
	static void setTransactionTraceInformation (const arm_id_t &tran_id, const arm4db_transaction_summary_t &summary);
	static void setNewTransactionTraceInformation (int index);

	// Operators

protected:
	static void lock_transaction (void);
	static void unlock_transaction (void);

	static arm4_shm_header_t *g_shared_memory_ptr;
	static int g_message_queue;
	static int g_umask;

	static pthread_once_t g_attached;

	// This is a library local lock
	static pthread_rwlock_t g_db_rw_lock;

#ifndef USE_POSIX_PROCESS_SEM
	static sem_t *g_sequence_sem_ptr;
	static sem_t *g_application_sem_ptr;
	static sem_t *g_transaction_sem_ptr;
	static sem_t *g_metric_sem_ptr;
	static sem_t *g_utility_sem_ptr;

	static const char *instanceName (const char *template_ptr);
#endif /* USE_POSIX_PROCESS_SEM */

private:

	static void attach_semaphores (void);
	static void doDetach (void);
	static void doAttach (void);
};

class Arm4dbLock :
	private Arm4dbSharedMemory
{
public:
	// Constructor
	Arm4dbLock ();

	// Destructor
	virtual ~Arm4dbLock ();

	// Accessors

	// Mutators

	// Operations
	void lock (void);
	void unlock (void);

	// Operators

private:
	bool m_locked;
};

class Arm4dbWriteLock :
	private Arm4dbSharedMemory
{
public:
	// Constructor
	Arm4dbWriteLock ();

	// Destructor
	virtual ~Arm4dbWriteLock ();

	// Accessors

	// Mutators

	// Operations
	void lock (void);
	void unlock (void);

	// Operators

private:
	bool m_locked;
};

class Arm4dbUtilityLock :
	private Arm4dbSharedMemory
{
public:
	// Constructor
	Arm4dbUtilityLock ();

	// Destructor
	virtual ~Arm4dbUtilityLock ();

	// Accessors

	// Mutators

	// Operations
	void lock (void);
	void unlock (void);

	// Operators

private:
	bool m_locked;
};

#endif /* ARM4DBSHAREDMEMORY_H */
