/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef BERKELEYDB_H	/* Prevent multiple inclusion */
#define BERKELEYDB_H 1

#include <db_cxx.h>
#include "Arm4dbBackEnd.h"

#define WRAP_EXCEPTION(status,command) \
    {\
    try { (status) = (command); } \
    catch (DbRunRecoveryException r) { panic (r, __FILE__, __LINE__); } \
    catch (DbException e) { status = e.get_errno (); }\
    }

#define WRAP_EXCEPTION_STEP(status,command,step) \
    {\
    try { (status) = (command); step++; } \
    catch (DbRunRecoveryException r) { panic (r, __FILE__, __LINE__); } \
    catch (DbException e) { status = e.get_errno (); }\
    }

#define MAX_FAILURES	(5)

/* The various databases */
#define SEQUENCE_DB_NAME							"arm4_sequence.db"

#define APPLICATION_DB_NAME							"arm4_appl.db"
#define APPLICATION_NAME_DB_NAME					"arm4_appl_name.db"
#define APPLICATION_IDENTITY_DB_NAME				"arm4_appl_identity.db"
#define APPLICATION_CONTEXT_NAMES_DB_NAME			"arm4_appl_context_names.db"
#define APPLICATION_INSTANCE_DB_NAME				"arm4_appl_instance.db"
#define APPLICATION_INSTANCE_APP_DB_NAME			"arm4_appl_instance_app.db"
#define APPLICATION_GROUP_DB_NAME					"arm4_appl_group.db"
#define APPLICATION_INSTANCE_NAMES_DB_NAME			"arm4_appl_instance_names.db"
#define APPLICATION_CONTEXT_VALUES_DB_NAME			"arm4_appl_context_values.db"
#define APPLICATION_ADDRESS_DB_NAME					"arm4_appl_address.db"

#define TRANSACTION_DB_NAME							"arm4_tran.db"
#define TRANSACTION_NAME_DB_NAME					"arm4_tran_name.db"
#define TRANSACTION_APPID_DB_NAME					"arm4_tran_appid.db"
#define TRANSACTION_IDENTITY_DB_NAME				"arm4_tran_identity.db"
#define TRANSACTION_CONTEXT_NAMES_DB_NAME			"arm4_tran_context_names.db"
#define TRANSACTION_IDENTITY_URI_DB_NAME			"arm4_tran_identity_uri.db"
#define TRANSACTION_METRIC_BINDING_DB_NAME			"arm4_tran_metric_binding.db"
#define TRANSACTION_INSTANCE_DB_NAME				"arm4_tran_instance.db"
#define TRANSACTION_INSTANCE_TRANID_DB_NAME			"arm4_tran_instance_tranid.db"
#define TRANSACTION_INSTANCE_APP_HANDLE_DB_NAME		"arm4_tran_instance_app_handle.db"
#define TRANSACTION_CONTEXT_VALUES_DB_NAME			"arm4_tran_context_values.db"
#define TRANSACTION_CONTEXT_URI_DB_NAME				"arm4_tran_context_uri.db"
#define TRANSACTION_CONTEXT_METRIC_VALUE_DB_NAME	"arm4_tran_context_metric_value.db"
#define TRANSACTION_CONTEXT_USER_DB_NAME			"arm4_tran_context_user.db"
#define TRANSACTION_CONTEXT_DIAG_DB_NAME			"arm4_tran_context_diag.db"

#define METRIC_DB_NAME                              "arm4_metric.db"
#define METRIC_NAME_DB_NAME					        "arm4_metric_name.db"
#define METRIC_APPID_DB_NAME					    "arm4_metric_appid.db"

#define CORRELATOR_PARENT_DB_NAME					"arm4_correlator_parent.db"
#define CORRELATOR_CURRENT_DB_NAME					"arm4_correlator_current.db"

#define TRACE_STAT_DB_NAME							"arm4_trace_stat.db"
#define APPDEX_STAT_DB_NAME							"arm4_appdex_stat.db"


class BerkeleyDB : public Arm4dbBackEnd
{
public:
	/* Constructors */
	BerkeleyDB ();

	/* Destructors */
	virtual ~BerkeleyDB ();

	/* Accessors */

	/* Mutators */

	/* Operations */
	void initializeLibrary (void);
	void initializeThread (void);

	arm_error_t openDatabases (void);
	arm_error_t openDatabasesReadOnly (void);
	arm_error_t closeDatabasesNoLock (void);

	arm_error_t initializeSequences (arm4_shm_header_t *header_ptr);

	arm_error_t processApplicationInstances (const arm4_shm_application_instance_t *app_instance_ptr);
	arm_error_t processApplicationGroup (const arm4_shm_property_t *group_ptr);
	arm_error_t processApplicationInstanceId (const arm4_shm_property_t *property_ptr);
	arm_error_t processApplicationContext (const arm4_shm_context_property_t *property_ptr);
	arm_error_t processApplicationAddress (const arm4_shm_address_t *address_ptr);
	arm_error_t processTransactionInstances (const arm4_shm_transaction_instance_t *tran_instance_ptr);
	arm_error_t processTransactionContext (const arm4_shm_context_property_t *property_ptr);
	arm_error_t processTransactionUri (const arm4_shm_uri_t *uri_ptr);
	arm_error_t processTransactionMetricValue (const arm4_shm_metric_value_t *metric_ptr);
	arm_error_t processTransactionUser (const arm4_shm_user_t *user_ptr);
	arm_error_t processTransactionDiag (const arm4_shm_diag_t *diag_ptr);
	arm_error_t processCorrelators (const arm4_shm_correlator_t *correlator_ptr);
	arm_error_t processSequence (const arm4_shm_db_sequence_t *sequence_ptr);
	arm_error_t processRegisterApplication (const arm4_shm_register_application_t *application_ptr,
		arm_id_t *appid_ptr,
		arm_boolean_t *is_new_ptr);
	arm_error_t processRegisterApplicationIdentity (const arm4_shm_register_application_identity_t *identity_ptr);
	arm_error_t processRegisterApplicationContext (const arm4_shm_register_application_context_t *context_ptr);
	arm_error_t processRegisterTransaction (const arm4_shm_register_transaction_t *transaction_ptr,
		arm_id_t *tranid_ptr,
		arm_boolean_t *is_new_ptr);
	arm_error_t processRegisterTransactionIdentity (const arm4_shm_register_application_identity_t *identity_ptr);
	arm_error_t processRegisterTransactionContext (const arm4_shm_register_application_context_t *context_ptr);
	arm_error_t processRegisterTransactionUri (const arm4_shm_register_transaction_uri_t *uri_ptr);
	arm_error_t processRegisterTransactionMetricBinding (const arm4_shm_register_transaction_metric_t *metric_ptr);
	arm_error_t processRegisterMetric (arm4_shm_register_metric_t *metric_ptr,
		arm_id_t *metricid_ptr);
	arm_error_t processTraceStats (arm_time_t sample_time, arm4db_transaction_trace_stats_t *stats_ptr, int transaction_count);

	void saveApplicationDefinition (const arm_id_t &appl_id, const application_record_t &application);
	void saveApplicationIdentity (const arm_id_t &appl_id, const property_record_t &property);
	void saveApplicationContext (const arm_id_t &appl_id, const indexed_property_name_record_t &property);

	void saveMetricDefinition (const arm_id_t &metric_id, const metric_record_t &metric);
	
	void saveTransactionDefinition (const arm_id_t &tran_id, const transaction_record_t &transaction);
	void saveTransactionIdentity (const arm_id_t &tran_id, const property_record_t &property);
	void saveTransactionContext (const arm_id_t &tran_id, const indexed_property_name_record_t &property);
	void saveTransactionUri (const arm_id_t &tran_id, const char *uri_ptr);
	void saveTransactionMetric (const arm_id_t &tran_id, const arm_metric_binding_t &binding);

	void saveApplicationInstance (const arm_app_start_handle_t app_handle, const application_instance_record_t &application);
	void saveAppInstanceContextValue (const arm_app_start_handle_t app_handle, const indexed_property_value_record_t &property);
	void saveAppInstanceGroup (const arm_app_start_handle_t app_handle, const char *group_ptr);
	void saveAppInstanceName (const arm_app_start_handle_t app_handle, const char *name_ptr);
	void saveAppInstanceAddress (const arm_app_start_handle_t app_handle, const application_address_record_t &address_value);

	void saveTransactionInstance (const arm_tran_start_handle_t tran_handle, const transaction_instance_record_t &transaction);
	void saveTranInstanceContextValue (const arm_tran_start_handle_t tran_handle, const indexed_property_value_record_t &property);
	void saveTranInstanceMetricValue (const arm_tran_start_handle_t tran_handle, const metric_value_record_t &metric);
	void saveTranInstanceUri (const arm_tran_start_handle_t tran_handle, const char *uri_ptr);
	void saveTranInstanceUser (const arm_tran_start_handle_t tran_handle, const user_record_t &user);
	void saveTranInstanceDiag (const arm_tran_start_handle_t tran_handle, const char *diag_ptr);
	void saveTranCorrelator (const arm_tran_start_handle_t tran_handle, const arm_correlator_t *correlator_ptr);
	void saveTranParentCorrelator (const arm_tran_start_handle_t tran_handle, const arm_correlator_t *correlator_ptr);

	void saveTraceStats (const arm_time_t sample_time, const arm4db_transaction_trace_stats_t &trace_stats);

	/* Checkpoint the database transactions */
	virtual void checkpoint (void);
	
	/* Get transaction stats */
	virtual void *getStats (void);

	/* Debugging routines */
	virtual void dump (void) const;
	
	/* Database backup and rollover */
	virtual void backup (void);

    /* Reporting functions */
    virtual void printSummaries (void) const;
    virtual void printTransactions (void) const;

    virtual std::list<arm_id_t> *getApplicationList (void);
    virtual std::list<arm_id_t> *getTransactionList (void);
    virtual std::list<arm_id_t> *getTransactionList (const arm_id_t &app_id);
	virtual std::list<arm_tran_start_handle_t> *getTransactionInstanceList (const arm_id_t &tran_id);
	virtual int getApplicationCount (void);
	virtual int getTransactionCount (void);
	virtual int getTransactionCount (const arm_id_t &app_id);
    
    virtual arm_error_t getApplicationInformation (const arm_id_t &app_id, arm4db_application_summary_t *summary_ptr);
    virtual arm_error_t getTransactionInformation (const arm_id_t &tran_id, arm4db_transaction_summary_t *summary_ptr);
	virtual const arm_correlator_t *getCorrelator (arm_tran_start_handle_t tran_handle, arm_correlator_t *correlator_ptr);
	virtual const arm_correlator_t *getParentCorrelator (arm_tran_start_handle_t tran_handle, arm_correlator_t *correlator_ptr);
    
    virtual void setTransactionTraceInformation (const arm_id_t &tran_id, const arm4db_transaction_summary_t &summary);
    
    virtual std::ostream& exportApplicationDefinitions (std::ostream &os) const;
    virtual std::ostream& exportTransactionDefinitions (std::ostream &os) const;
    virtual std::ostream& exportMetricDefinitions (std::ostream &os) const;
    virtual std::ostream& exportApplications (std::ostream &os) const;
    virtual std::ostream& exportTransactions (std::ostream &os) const;
    virtual std::ostream& exportTraceStats (std::ostream &os) const;

	/* Operators */

private:
	/* Mark these as private to prevent their use */
	BerkeleyDB (const Arm4db &);
	BerkeleyDB &operator= (const Arm4db &);

	const Arm4dbConfig &getConfig (void) { return Arm4dbConfig::getConfig (); }

    static void panic (const DbRunRecoveryException &r, const char *file, int line);
    static void panic (const DbException &e, const char *file, int line);

	int openEnvironment (int flags);
	int openDatabaseWithTxn (Db **db_ptr,
		const char *db_name_ptr,
		int flags,
		DbTxn *transaction_ptr);
    int openDatabase (Db **db_ptr,
    	const char *db_name_ptr,
    	int flags);
    void close_database (Db **database_pptr);

	/* Association methods */
	static int get_application_name (Db *db_ptr,
		const Dbt *primary_key_ptr,
		const Dbt *primary_data_ptr,
		Dbt *secondary_key_ptr);
	static int get_application_instance_app (Db *db_ptr,
		const Dbt *primary_key_ptr,
		const Dbt *primary_data_ptr,
		Dbt *secondary_key_ptr);
	static int get_transaction_name (Db *db_ptr,
		const Dbt *primary_key_ptr,
		const Dbt *primary_data_ptr,
		Dbt *secondary_key_ptr);
	static int get_transaction_appid (Db *db_ptr,
		const Dbt *primary_key_ptr,
		const Dbt *primary_data_ptr,
		Dbt *secondary_key_ptr);
	static int get_tran_instance_tranid (Db *db_ptr,
		const Dbt *primary_key_ptr,
		const Dbt *primary_data_ptr,
		Dbt *secondary_key_ptr);
	static int get_tran_instance_app_handle (Db *db_ptr,
		const Dbt *primary_key_ptr,
		const Dbt *primary_data_ptr,
		Dbt *secondary_key_ptr);
	static int get_metric_name (Db *db_ptr,
		const Dbt *primary_key_ptr,
		const Dbt *primary_data_ptr,
		Dbt *secondary_key_ptr);
	static int get_metric_appid (Db *db_ptr,
		const Dbt *primary_key_ptr,
		const Dbt *primary_data_ptr,
		Dbt *secondary_key_ptr);

	int openApplicationDatabase (int flags);
	int openApplicationInstanceDatabase (int flags);
	int openTransactionDatabase (int flags);
	int openTransactionInstanceDatabase (int flags);
	int openMetricDatabase (int flags);
	arm_error_t openDatabasesNoLock (int flags);

	int	process_application_instances_txn (const arm4_shm_application_instance_t *entry_ptr);
	int process_application_group_txn (const arm4_shm_property_t *group_ptr);
	int process_application_instance_id_txn (const arm4_shm_property_t *property_ptr);
	int process_application_context_txn (const arm4_shm_context_property_t *property_ptr);
	int process_application_address_txn (const arm4_shm_address_t *address_ptr);
	int process_transaction_instances_txn (const arm4_shm_transaction_instance_t *entry_ptr);
	int process_transaction_context_txn (const arm4_shm_context_property_t *property_ptr);
	int process_transaction_uri_txn (const arm4_shm_uri_t *property_ptr);
	int process_transaction_metric_value_txn (const arm4_shm_metric_value_t *metric_ptr);
	int process_transaction_user_txn (const arm4_shm_user_t *user_ptr);
	int process_transaction_diag_txn (const arm4_shm_diag_t *diag_ptr);
	int process_correlator_txn (const arm4_shm_correlator_t *correlator_ptr);
	int process_sequence_txn (const arm4_shm_db_sequence_t *sequence_ptr);
	int process_register_application_txn (const unsigned char *digest_ptr,
		const char *application_name_ptr,
		const arm_charset_t charset,
		arm_id_t *output_app_id,
		arm_boolean_t *is_new_ptr);
	int process_register_application_identity_txn (const arm4_shm_register_application_identity_t *identity_ptr);
	int process_register_application_context_txn (const arm4_shm_register_application_context_t *context_ptr);
	int process_register_transaction_txn (const arm4_shm_register_transaction_t *transaction_ptr,
		arm_id_t *output_tran_id,
		arm_boolean_t *is_new_ptr);
	int process_register_transaction_identity_txn (const arm4_shm_register_application_identity_t *identity_ptr);
	int process_register_transaction_context_txn (const arm4_shm_register_application_context_t *context_ptr);
	int process_register_transaction_uri_txn (const arm4_shm_register_transaction_uri_t *uri_ptr);
	int process_register_transaction_metric_binding_txn (const arm4_shm_register_transaction_metric_t *metric_ptr);
	int process_register_metric_txn (const unsigned char *digest_ptr,
		const char *metric_name_ptr,
		const arm_id_t *app_id,
		const char *metric_units_ptr,
		const arm_metric_format_t metric_format,
		const arm_metric_usage_t metric_usage,
		arm_id_t *output_metric_id);
	int process_trace_stats_txn (arm_time_t sample_time, arm4db_transaction_trace_stats_t *stats_ptr);

	int initialize_sequence (int index, arm4_shm_header_t *header_ptr);

	void dump_sequence (void) const;
	void dump_application (void) const;
	void dump_application_identity (void) const;
	void dump_application_context_names (void) const;
	void dump_application_instance (void) const;
	void dump_application_group (void) const;
	void dump_application_instance_names (void) const;
	void dump_application_context_values (void) const;
	void dump_application_address (void) const;
	void dump_transaction (void) const;
	void dump_transaction_identity (void) const;
	void dump_transaction_context_names (void) const;
	void dump_transaction_identity_uri (void) const;
	void dump_transaction_metric_binding (void) const;
	void dump_transaction_instance (void) const;
	void dump_transaction_context_values (void) const;
	void dump_transaction_context_uri (void) const;
	void dump_transaction_context_metric_value (void) const;
	void dump_transaction_context_user (void) const;
	void dump_transaction_context_diag (void) const;
	void dump_metric (void) const;

    std::ostream& export_application_definition (std::ostream &os, const arm_id_t &appl_id, const application_record_t &application) const;
	std::ostream& export_application_identity (std::ostream &os, const arm_id_t &appl_id) const;
	std::ostream& export_application_context_names (std::ostream &os, const arm_id_t &appl_id) const;

	std::ostream& export_transaction_identity (std::ostream &os, const arm_id_t &tran_id) const;
	std::ostream& export_transaction_context (std::ostream &os, const arm_id_t &tran_id) const;
	std::ostream& export_transaction_uri (std::ostream &os, const arm_id_t &tran_id) const;
	std::ostream& export_transaction_metric_binding (std::ostream &os, const arm_id_t &tran_id) const;
	
	std::ostream& exportApplicationGroup (std::ostream &os, arm_app_start_handle_t app_handle) const;
	std::ostream& exportApplicationInstanceNames (std::ostream &os, arm_app_start_handle_t app_handle) const;
	std::ostream& exportApplicationContextValues (std::ostream &os, arm_app_start_handle_t app_handle) const;
	std::ostream& exportApplicationAddress (std::ostream &os, arm_app_start_handle_t app_handle) const;

	std::ostream& exportTransactionContextValues (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportTransactionContextUri (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportTransactionContextMetricValue (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportTransactionContextUser (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportTransactionContextDiag (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportParentCorrelator (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportCorrelator (std::ostream &os, arm_tran_start_handle_t tran_handle) const;

	arm_error_t copyDatabases (void);
	void destroyInstanceDatabase (const char *name_ptr);
	void destroyInstanceDatabases (void);

    void find_and_print_correlators (arm_tran_start_handle_t tran_handle) const;
    void find_and_print_metric_values (arm_tran_start_handle_t tran_handle) const;
    void find_and_print_user_values (arm_tran_start_handle_t tran_handle) const;
    void find_and_print_diag_values (arm_tran_start_handle_t tran_handle) const;
    int find_application (application_record_t *application_ptr,
    	const arm_id_t *appl_id) const;
    int find_transaction (transaction_record_t *transaction_ptr,
    	const arm_id_t *transaction_instance_ptr) const;
    void print_transaction_metadata (const arm_id_t *tran_id) const;
	void print_trace_stats (const arm_id_t *tran_id) const;
    arm_error_t summarize_transaction (const arm_id_t *tran_id) const;
    void print_application_metadata (const arm_id_t *appl_id) const;
    arm_error_t summarize_application (const arm_id_t *appl_id, const application_record_t *application_ptr) const;

	int save (Db *db_ptr, const void *key_ptr, size_t key_size, const void *data_ptr, size_t data_size);
	int save (Db *db_ptr, const void *key_ptr, size_t key_size, const char *data_ptr);

    DbEnv *m_environmentPtr;

	/* The database objects */
	Db	*db_sequence_ptr;

	Db	*db_application_ptr;
	Db	*db_application_name_ptr;
	Db	*db_application_identity_ptr;
	Db	*db_application_context_names_ptr;
	Db	*db_application_instance_ptr;
	Db	*db_application_instance_app_ptr;
	Db	*db_application_group_ptr;
	Db	*db_application_instance_names_ptr;
	Db	*db_application_context_values_ptr;
	Db	*db_application_address_ptr;
	
	Db	*db_transaction_ptr;
	Db	*db_transaction_name_ptr;
	Db	*db_transaction_appid_ptr;
	Db	*db_transaction_identity_ptr;
	Db	*db_transaction_context_names_ptr;
	Db	*db_transaction_identity_uri_ptr;
	Db	*db_transaction_metric_binding_ptr;
	Db	*db_transaction_instance_ptr;
	Db	*db_transaction_instance_tranid_ptr;
	Db	*db_transaction_instance_app_handle_ptr;
	Db	*db_transaction_context_values_ptr;
	Db	*db_transaction_context_uri_ptr;
	Db	*db_transaction_context_metric_value_ptr;
	Db	*db_transaction_context_user_ptr;
	Db	*db_transaction_context_diag_ptr;
	
	Db  *db_metric_ptr;
	Db	*db_metric_name_ptr;
	Db	*db_metric_appid_ptr;
	
	Db	*db_parent_correlator_ptr;
	Db	*db_current_correlator_ptr;
	
	Db	*db_trace_stat_ptr;
	Db	*db_appdex_stat_ptr;
};

#endif /* BERKELEYDB_H */
