/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef LIBARM4DB_LOGGER_H	/* Prevent multiple inclusion */
#define LIBARM4DB_LOGGER_H 1

#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <string>
#include "Arm4dbExceptions.h"

class Logger
{
public:
	/* Constructors */

	/* Destructors */
	virtual ~Logger ();

	/* Accessors */
    static Logger &getLogger (void); // Singleton accessor

	/* Mutators */
    void setFile (FILE *filePtr) { m_filePtr = filePtr; }
	void setFile (const std::string &name);
    void setName (const std::string &name) { m_namePtr = name; }

	/* Operations */
    void log (const char *format, ...) const;
    void error (const char *format, ...) const; // Log the system error number as well
	void exceptionError (const Arm4dbException &e) const;
	void exceptionError (const Arm4dbException &e, const char *format, ...) const;

    /* Operators */

private:
	/* Mark these as private to prevent their use */
	Logger ();
	Logger (const Logger &);
	Logger &operator= (const Logger &);

    void openSyslog (void) const;

    static Logger *m_loggerPtr; // Singleton
    FILE *m_filePtr;
    std::string m_namePtr;
};

#endif /* LIBARM4DB_LOGGER_H */

