/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef SQLITE_H	/* Prevent multiple inclusion */
#define SQLITE_H 1

#include <list>
#include <sqlite3.h>

#include "Arm4dbBackEnd.h"

#define SQLITE_DB_NAME							"arm4.db"
 
class Sqlite : public Arm4dbBackEnd
{
public:
	/* Constructors */
	Sqlite ();

	/* Destructors */
	virtual ~Sqlite ();

	/* Accessors */

	/* Mutators */

	/* Operations */
	void initializeLibrary (void);
	void initializeThread (void);

	arm_error_t openDatabases (void);
	arm_error_t openDatabasesReadOnly (void);
	arm_error_t closeDatabasesNoLock (void);

	arm_error_t initializeSequences (arm4_shm_header_t *header_ptr);

	arm_error_t processApplicationInstances (const arm4_shm_application_instance_t *app_instance_ptr);
	arm_error_t processApplicationGroup (const arm4_shm_property_t *group_ptr);
	arm_error_t processApplicationInstanceId (const arm4_shm_property_t *property_ptr);
	arm_error_t processApplicationContext (const arm4_shm_context_property_t *property_ptr);
	arm_error_t processApplicationAddress (const arm4_shm_address_t *address_ptr);
	arm_error_t processTransactionInstances (const arm4_shm_transaction_instance_t *tran_instance_ptr);
	arm_error_t processTransactionContext (const arm4_shm_context_property_t *property_ptr);
	arm_error_t processTransactionUri (const arm4_shm_uri_t *uri_ptr);
	arm_error_t processTransactionMetricValue (const arm4_shm_metric_value_t *metric_ptr);
	arm_error_t processTransactionUser (const arm4_shm_user_t *user_ptr);
	arm_error_t processTransactionDiag (const arm4_shm_diag_t *diag_ptr);
	arm_error_t processCorrelators (const arm4_shm_correlator_t *correlator_ptr);
	arm_error_t processSequence (const arm4_shm_db_sequence_t *sequence_ptr);
	arm_error_t processRegisterApplication (const arm4_shm_register_application_t *application_ptr,
		arm_id_t *appid_ptr,
		arm_boolean_t *is_new_ptr);
	arm_error_t processRegisterApplicationIdentity (const arm4_shm_register_application_identity_t *identity_ptr);
	arm_error_t processRegisterApplicationContext (const arm4_shm_register_application_context_t *context_ptr);
	arm_error_t processRegisterTransaction (const arm4_shm_register_transaction_t *transaction_ptr,
		arm_id_t *tranid_ptr,
		arm_boolean_t *is_new_ptr);
	arm_error_t processRegisterTransactionIdentity (const arm4_shm_register_application_identity_t *identity_ptr);
	arm_error_t processRegisterTransactionContext (const arm4_shm_register_application_context_t *context_ptr);
	arm_error_t processRegisterTransactionUri (const arm4_shm_register_transaction_uri_t *uri_ptr);
	arm_error_t processRegisterTransactionMetricBinding (const arm4_shm_register_transaction_metric_t *metric_ptr);
	arm_error_t processRegisterMetric (arm4_shm_register_metric_t *metric_ptr,
		arm_id_t *metricid_ptr);
	arm_error_t processTraceStats (arm_time_t sample_time, arm4db_transaction_trace_stats_t *stats_ptr, int transaction_count);

	void saveApplicationDefinition (const arm_id_t &appl_id, const application_record_t &application);
	void saveApplicationIdentity (const arm_id_t &appl_id, const property_record_t &property);
	void saveApplicationContext (const arm_id_t &appl_id, const indexed_property_name_record_t &property);

	void saveMetricDefinition (const arm_id_t &metric_id, const metric_record_t &metric);

	void saveTransactionDefinition (const arm_id_t &tran_id, const transaction_record_t &transaction);
	void saveTransactionIdentity (const arm_id_t &tran_id, const property_record_t &property);
	void saveTransactionContext (const arm_id_t &tran_id, const indexed_property_name_record_t &property);
	void saveTransactionUri (const arm_id_t &tran_id, const char *uri_ptr);
	void saveTransactionMetric (const arm_id_t &tran_id, const arm_metric_binding_t &binding);

	void saveApplicationInstance (const arm_app_start_handle_t app_handle, const application_instance_record_t &application);
	void saveAppInstanceContextValue (const arm_app_start_handle_t app_handle, const indexed_property_value_record_t &property);
	void saveAppInstanceGroup (const arm_app_start_handle_t app_handle, const char *group_ptr);
	void saveAppInstanceName (const arm_app_start_handle_t app_handle, const char *name_ptr);
	void saveAppInstanceAddress (const arm_app_start_handle_t app_handle, const application_address_record_t &address_value);

	void saveTransactionInstance (const arm_tran_start_handle_t tran_handle, const transaction_instance_record_t &transaction);
	void saveTranInstanceContextValue (const arm_tran_start_handle_t tran_handle, const indexed_property_value_record_t &property);
	void saveTranInstanceMetricValue (const arm_tran_start_handle_t tran_handle, const metric_value_record_t &metric);
	void saveTranInstanceUri (const arm_tran_start_handle_t tran_handle, const char *uri_ptr);
	void saveTranInstanceUser (const arm_tran_start_handle_t tran_handle, const user_record_t &user);
	void saveTranInstanceDiag (const arm_tran_start_handle_t tran_handle, const char *diag_ptr);
	void saveTranCorrelator (const arm_tran_start_handle_t tran_handle, const arm_correlator_t *correlator_ptr);
	void saveTranParentCorrelator (const arm_tran_start_handle_t tran_handle, const arm_correlator_t *correlator_ptr);

	void saveTraceStats (const arm_time_t sample_time, const arm4db_transaction_trace_stats_t &trace_stats);

	/* Checkpoint the database transactions */
	virtual void checkpoint (void);
	
	/* Get transaction stats */
	virtual void *getStats (void);

	/* Debugging routines */
	virtual void dump (void) const;
	
	/* Database backup and rollover */
	virtual void backup (void);

	/* Reporting functions */
    virtual void printSummaries (void) const;
    virtual void printTransactions (void) const;

    virtual std::list<arm_id_t> *getApplicationList (void);
    virtual std::list<arm_id_t> *getTransactionList (void);
    virtual std::list<arm_id_t> *getTransactionList (const arm_id_t &app_id);
	virtual std::list<arm_tran_start_handle_t> *getTransactionInstanceList (const arm_id_t &tran_id);
	virtual int getApplicationCount (void);
	virtual int getTransactionCount (void);
	virtual int getTransactionCount (const arm_id_t &app_id);
    
    virtual arm_error_t getApplicationInformation (const arm_id_t &app_id, arm4db_application_summary_t *summary_ptr);
    virtual arm_error_t getTransactionInformation (const arm_id_t &tran_id, arm4db_transaction_summary_t *summary_ptr);
	virtual const arm_correlator_t *getCorrelator (arm_tran_start_handle_t tran_handle, arm_correlator_t *correlator_ptr);
	virtual const arm_correlator_t *getParentCorrelator (arm_tran_start_handle_t tran_handle, arm_correlator_t *correlator_ptr);
    
    virtual void setTransactionTraceInformation (const arm_id_t &tran_id, const arm4db_transaction_summary_t &summary);
    
    virtual std::ostream& exportApplicationDefinitions (std::ostream &os) const;
    virtual std::ostream& exportTransactionDefinitions (std::ostream &os) const;
    virtual std::ostream& exportMetricDefinitions (std::ostream &os) const;
    virtual std::ostream& exportApplications (std::ostream &os) const;
    virtual std::ostream& exportTransactions (std::ostream &os) const;
    virtual std::ostream& exportTraceStats (std::ostream &os) const;

	/* Operators */

private:
	/* Mark these as private to prevent their use */
	Sqlite (const Arm4db &);
	Sqlite &operator= (const Arm4db &);
	
	/* The active database handle */
	sqlite3 *m_db;

	const Arm4dbConfig &getConfig (void) { return Arm4dbConfig::getConfig (); }

	arm_error_t openDatabasesReadOnlyNoLock (void);
	arm_error_t openDatabasesNoLock (void);
	arm_error_t openOrCreateDatabasesNoLock (void);
	arm_error_t initializeSequence (int index, arm4_shm_header_t *header_ptr);
	
	void logError (void) const;
	void logError (const char *error_point) const;
	void logError (const std::string &error_point) const { logError (error_point.c_str ()); }

	std::string fullDbName (void);
	bool dbExists (void);

	std::ostream& exportApplicationIdentity (std::ostream &os, const arm_id_t &appl_id) const;
	std::ostream& exportApplicationContext (std::ostream &os, const arm_id_t &appl_id) const;
	
	std::ostream& exportTransactionIdentity (std::ostream &os, const arm_id_t &tran_id) const;
	std::ostream& exportTransactionContext (std::ostream &os, const arm_id_t &tran_id) const;
	std::ostream& exportTransactionUri (std::ostream &os, const arm_id_t &tran_id) const;
	std::ostream& exportTransactionMetricBinding (std::ostream &os, const arm_id_t &tran_id) const;

	std::ostream& exportApplicationGroup (std::ostream &os, arm_app_start_handle_t app_handle) const;
	std::ostream& exportApplicationInstanceNames (std::ostream &os, arm_app_start_handle_t app_handle) const;
	std::ostream& exportApplicationContextValues (std::ostream &os, arm_app_start_handle_t app_handle) const;
	std::ostream& exportApplicationAddress (std::ostream &os, arm_app_start_handle_t app_handle) const;

	std::ostream& exportTransactionContextValues (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportTransactionContextUri (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportTransactionContextMetricValue (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportTransactionContextUser (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportTransactionContextDiag (std::ostream &os, arm_tran_start_handle_t tran_handle) const;
	std::ostream& exportCorrelators (std::ostream &os, arm_tran_start_handle_t tran_handle) const;

#if defined(HAVE_SQLITE_PREPARE_V2)
	static int prepare(sqlite3 *db, const char *zSql, int nByte, sqlite3_stmt **ppStmt, const char **pzTail)
		{ return sqlite3_prepare_v2 (db, zSql, nByte, ppStmt, pzTail); }
#else
	static int prepare(sqlite3 *db, const char *zSql, int nByte, sqlite3_stmt **ppStmt, const char **pzTail)
		{ return sqlite3_prepare (db, zSql, nByte, ppStmt, pzTail); }
#endif
};

#endif /* SQLITE_H */
