/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef ARM4_DB_H	/* Prevent multiple inclusion */
#define ARM4_DB_H 1

/* Database record structures */
typedef struct property_record {
	arm_char_t name [ARM_PROPERTY_NAME_MAX_LENGTH];
	arm_char_t value [ARM_PROPERTY_VALUE_MAX_LENGTH];
} property_record_t;

typedef struct indexed_property_name_record {
	int index;
	arm_char_t name [ARM_PROPERTY_NAME_MAX_LENGTH];
} indexed_property_name_record_t;

typedef struct indexed_property_value_record {
	int index;
	arm_char_t value [ARM_PROPERTY_VALUE_MAX_LENGTH];
} indexed_property_value_record_t;

typedef struct handle_list_record {
	arm_int64_t handle;
	struct handle_list_record *next_ptr;
} handle_list_record_t;

typedef struct application_record {
	arm_char_t application_name [ARM_NAME_MAX_LENGTH];
	char md5_digest [MD5_DIGEST_LENGTH];
    arm_charset_t charset;
	arm4_shm_transaction_trace_t default_trace_flags;
	arm_int64_t default_trace_parameter;
	arm_int64_t default_appdex_threshold;
} application_record_t;

typedef struct application_instance_record {
	arm_id_t appl_id;
	arm_boolean_t active;
} application_instance_record_t;

typedef struct application_address_record {
	arm_int16_t address_format;
	arm_int16_t address_length;
	arm_uint8_t address [ARM_PROPERTY_URI_MAX_LENGTH];
	arm_boolean_t id_valid;
	arm_id_t id;
} application_address_record_t;

typedef struct transaction_record {
	arm_char_t transaction_name [ARM_NAME_MAX_LENGTH];
	char md5_digest [MD5_DIGEST_LENGTH];
	arm_id_t appl_id;
	arm4_shm_transaction_trace_t trace_flags;
	arm4_appdex_t appdex_flags;
	arm_int64_t trace_parameter;
	arm_int64_t appdex_threshold;
} transaction_record_t;

/*
 * This record is used for the in-memory records by the libarm library, as well as
 * by the Berkeley DB.
 */

typedef struct transaction_instance_record {
	arm_id_t appl_id;
	arm_id_t tran_id;
	arm_app_start_handle_t app_handle;
	arm_time_t arrival_time;
	arm_time_t stop_time;
	arm_time_t response_time;
	arm_tran_status_t tran_status;
	arm_boolean_t active;
	arm_time_t block_start_time;
	arm_time_t blocked_time;
} transaction_instance_record_t;

typedef struct metric_record {
	arm_char_t metric_name [ARM_NAME_MAX_LENGTH];
	arm_char_t metric_units [ARM_NAME_MAX_LENGTH];
	char md5_digest [MD5_DIGEST_LENGTH];
	arm_id_t appl_id;
    arm_metric_format_t metric_format;
    arm_metric_usage_t metric_usage;
} metric_record_t;

typedef struct metric_value_record
{
	arm_metric_t value;
    arm_char_t string32 [ARM_METRIC_STRING32_MAX_LENGTH];
} metric_value_record_t;

typedef struct user_record
{
	char name [ARM_PROPERTY_NAME_MAX_LENGTH];
    arm_boolean_t id_valid;
    arm_id_t id;
} user_record_t;

#ifdef __cplusplus

#include <list>

typedef struct arm4db_application_summary {
	arm_id_t appl_id;
	arm_char_t application_name [ARM_NAME_MAX_LENGTH];
    arm_charset_t charset;
	std::list<property_record_t *> identity_properties;
	std::list<indexed_property_name_record_t *> context_names;
	arm4_shm_transaction_trace_t default_trace_flags;
	arm_int64_t default_trace_parameter;
	arm_int64_t default_appdex_threshold;
} arm4db_application_summary_t;

typedef struct arm4db_transaction_summary {
	arm_id_t tran_id;
	arm_id_t appl_id;
	arm_char_t transaction_name [ARM_NAME_MAX_LENGTH];
	std::list<property_record_t *> identity_properties;
	std::list<indexed_property_name_record_t *> context_names;
	arm_char_t uri [ARM_PROPERTY_URI_MAX_LENGTH];
	arm4_shm_transaction_trace_t trace_flags;
	arm4_appdex_t appdex_flags;
	arm_int64_t trace_parameter;
	arm_int64_t appdex_threshold;
} arm4db_transaction_summary_t;

typedef struct arm4db_transaction_trace_stats {
	arm_id_t tran_id;
	arm_int64_t transaction_counter;
} arm4db_transaction_trace_stats_t;

typedef struct arm4db_appdex_stats {
	arm_id_t tran_id;
	arm_int64_t appdex_satisfactory;
	arm_int64_t appdex_tolerable;
	arm_int64_t appdex_frustrating;
} arm4db_appdex_stats_t;

#endif // __cplusplus

#endif /* ARM4_DB_H */
