/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#ifndef LIBARM4_SHM_H
#define LIBARM4_SHM_H 1

#include "arm4_shm.h"

int __libarm4_shm_get_instance (void);

arm4_shm_header_t *__libarm4_shm_get_header (void);
arm_boolean_t __libarm4_shm_is_collecting (void);

/* Tree mutexes. These always use pthreads, since they're local to the process */
struct tree; /* Forward declaration */
void __libarm4_shm_init_tree (struct tree *tree_ptr);
void __libarm4_shm_lock_tree (struct tree *tree_ptr);
void __libarm4_shm_unlock_tree (struct tree *tree_ptr);

/* Mutex routines */
void __libarm4_shm_lock_sequence (void);
void __libarm4_shm_unlock_sequence (void);
void __libarm4_shm_lock_application (void);
void __libarm4_shm_unlock_application (void);
void __libarm4_shm_lock_transaction (void);
void __libarm4_shm_unlock_transaction (void);
void __libarm4_shm_lock_metric (void);
void __libarm4_shm_unlock_metric (void);

/* Utility functions */
int __libarm4_shm_is_valid_identifier (const arm_char_t *name_ptr);

/* Application instance queue */
void __libarm4_shm_add_app_instance (arm_app_start_handle_t app_handle,
	arm_id_t appl_id,
	arm_boolean_t active);
void __libarm4_shm_add_app_group (arm_app_start_handle_t app_handle,
	const char *group_name);
void __libarm4_shm_add_app_instance_id (arm_app_start_handle_t app_handle,
	const char *id_ptr);
void __libarm4_shm_add_app_context (arm_app_start_handle_t app_handle,
	int index,
	const char *context_ptr);
void __libarm4_shm_add_app_address (arm_app_start_handle_t app_handle,
	const arm_subbuffer_system_address_t *address_buffer);

/* Transaction instance queue */
int __libarm4_shm_transaction_trace (const arm_id_t tran_id, const arm_int32_t flags, const arm_boolean_t force_trace);
int __libarm4_shm_try_tran_instance (arm_tran_start_handle_t tran_handle,
	arm_id_t appl_id,
	arm_id_t tran_id,
	arm_app_start_handle_t app_handle,
	arm_time_t arrival_time,
	arm_time_t stop_time,
	arm_time_t response_time,
	arm_time_t blocked_time,
	arm_tran_status_t tran_status,
	arm_boolean_t active);
void __libarm4_shm_add_tran_instance (arm_tran_start_handle_t tran_handle,
	arm_id_t appl_id,
	arm_id_t tran_id,
	arm_app_start_handle_t app_handle,
	arm_time_t arrival_time,
	arm_time_t stop_time,
	arm_time_t response_time,
	arm_time_t blocked_time,
	arm_tran_status_t tran_status,
	arm_boolean_t active);
void __libarm4_shm_add_tran_context (arm_tran_start_handle_t tran_handle,
	int index,
	const char *context_ptr);
void __libarm4_shm_add_tran_uri (arm_tran_start_handle_t tran_handle,
	const char *uri_ptr);
void __libarm4_shm_add_metric_value (arm_tran_start_handle_t tran_handle,
	const arm_metric_t *metric_ptr);
void __libarm4_shm_add_user (arm_tran_start_handle_t tran_handle,
	const arm_subbuffer_user_t *user_ptr);
void __libarm4_shm_add_diag (arm_tran_start_handle_t tran_handle,
	const arm_subbuffer_diag_detail_t *diag_ptr);

/* Correlator queue */
void __libarm4_shm_add_correlators (arm_tran_start_handle_t tran_handle,
	const arm_correlator_t *parent_correlator,
	const arm_correlator_t *current_correlator);

/* Sequences */
void __libarm4_shm_save_sequence (arm4_shm_sequence_t index,
	arm_int64_t sequence);
int __libarm4_shm_next_sequence_number (arm4_shm_sequence_t index,
	arm_int64_t *sequence_ptr);

/* Return requests */
void __libarm4_shm_return_request (void);
int __libarm4_shm_get_return (arm_int64_t *value_ptr);

/* Register application */
void __libarm4_shm_register_application (const unsigned char *digest_ptr,
	const arm_char_t *app_name_ptr,
	arm_charset_t charset,
	int application_index);
arm_error_t __libarm4_shm_register_application_identity (const arm_id_t *app_id_ptr,
	const arm_subbuffer_app_identity_t *identity_buffer_ptr);

/* Register transaction */
void __libarm4_shm_register_transaction (const unsigned char *digest,
	const arm_id_t *app_id_ptr,
	const arm_char_t *tran_name,
	int transaction_index);
arm_error_t __libarm4_shm_register_transaction_identity (const arm_id_t *tran_id_ptr,
	const arm_subbuffer_tran_identity_t *identity_buffer_ptr);
arm_error_t __libarm4_shm_register_transaction_metric_binding (const arm_id_t *tran_id_ptr,
	const arm_subbuffer_metric_bindings_t *metric_buffer_ptr);

/* Register metric */
void __libarm4_shm_register_metric (unsigned char *digest_ptr,
    const arm_id_t *app_id_ptr,
	const char *metric_name_ptr,
    const arm_metric_format_t metric_format,
    const arm_metric_usage_t metric_usage,
    const arm_char_t *metric_unit_ptr,
	const int metric_index);

void __libarm4_shm_get_uuid (arm_id_t *uuid_ptr);

void __libarm4_shm_trace_arm (const char *string_ptr);
void __libarm4_shm_trace_arm_internal (const char *string_ptr);
void __libarm4_shm_trace_arm_shm (const char *string_ptr);
void __libarm4_shm_trace_arm_shm_internal (const char *string_ptr);

#endif /* LIBARM4_SHM_H */
