/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

/* This is a specialized tree indexed using 64-bit handles */

#ifndef LIBARM4_TREE_H
#define LIBARM4_TREE_H 1

#include "arm4.h"
#include "libarm4_shm.h"
 
typedef int (*tree_compare_t) (const void *key1_ptr, const void *key2_ptr);
typedef void * (*tree_clone_t) (const void *key_ptr);
 
/* Tree record structures */
typedef struct tree_record {
	struct tree_record *left_ptr;
	struct tree_record *right_ptr;
	void *key_ptr;
	void *data_ptr;
} tree_record_t;

struct tree {
	tree_record_t	root;
	tree_compare_t	compare;
	tree_clone_t	clone;
	pthread_mutex_t	mutex;
};

typedef struct tree tree_t;
typedef struct tree tree_id_t;

/* Functions */
void __libarm4_tree_init (tree_t *tree_ptr);
void __libarm4_tree_insert (tree_t *tree_ptr, arm_int64_t key, void *data_ptr);
void *__libarm4_tree_find (tree_t *tree_ptr, arm_int64_t key);
void *__libarm4_tree_remove (tree_t *tree_ptr, arm_int64_t key);
void *__libarm4_tree_remove_next (tree_t *tree_ptr, arm_int64_t *key_ptr);

void __libarm4_tree_id_init (tree_id_t *tree_ptr);
void __libarm4_tree_id_insert (tree_id_t *tree_ptr, const arm_id_t *key_ptr, void *data_ptr);
void *__libarm4_tree_id_find (tree_id_t *tree_ptr, const arm_id_t *key_ptr);
void *__libarm4_tree_id_remove (tree_id_t *tree_ptr, const arm_id_t *key_ptr);
void *__libarm4_tree_id_remove_next (tree_id_t *tree_ptr, arm_id_t *key_ptr);

typedef struct application_instance_tree_record {
	arm_id_t appl_id;
	arm_boolean_t active;
	tree_t transaction_tree;

	/* Need to save system address information when provided */
	arm_int16_t address_format;
	arm_int16_t address_length;
	arm_uint8_t *address;
} application_instance_tree_record_t;

typedef struct alias_tree_record {
	arm_id_t actual_id;
	unsigned char digest [MD5_DIGEST_LENGTH];
} alias_tree_record_t;

#endif /* LIBARM4_TREE_H */
