/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "arm4.h"
#include "arm4agent.h"

static void
set_empty_id (arm_id_t *id_ptr)
{
	memset (id_ptr, 0, sizeof (arm_id_t));
}

ARM4_API_DYNAMIC(arm_error_t)
arm_register_application(
    const arm_char_t *app_name,
    const arm_id_t *input_app_id,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4,
    arm_id_t *output_app_id)
{
	if (output_app_id == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;

	set_empty_id (output_app_id);
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_destroy_application(
    const arm_id_t *app_id,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_register_transaction(
    const arm_id_t *app_id,
    const arm_char_t *tran_name,
    const arm_id_t *input_tran_id,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4,
    arm_id_t *output_tran_id)
{
	if (output_tran_id == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;

	set_empty_id (output_tran_id);
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_register_metric(
    const arm_id_t *app_id,
    const arm_char_t *metric_name,
    const arm_metric_format_t metric_format,
    const arm_metric_usage_t metric_usage,
    const arm_char_t *metric_unit,
    const arm_id_t *input_metric_id,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4,
    arm_id_t *output_metric_id)
{
    if (output_metric_id == NULL)
        return ARM_FAILURE_NULL_ARGUMENT;

    set_empty_id (output_metric_id);
	return ARM_SUCCESS;
}

/* application instance API functions */
ARM4_API_DYNAMIC(arm_error_t)
arm_start_application(
    const arm_id_t *app_id,
    const arm_char_t *app_group_ptr,
    const arm_char_t *app_instance_ptr,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4,
    arm_app_start_handle_t *app_handle)
{
	if (app_handle == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;

	/* set empty handle */
	*app_handle = 0;
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_stop_application(
    const arm_app_start_handle_t app_handle,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	return ARM_SUCCESS;
}

/* transaction instance API functions */
ARM4_API_DYNAMIC(arm_error_t)
arm_start_transaction(
    const arm_app_start_handle_t app_handle,
    const arm_id_t *tran_id,
    const arm_correlator_t *parent_correlator,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4,
    arm_tran_start_handle_t *tran_handle,
    arm_correlator_t *current_correlator)
{
	if (tran_handle == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;

	/* Initialize the handle as empty */
	*tran_handle = 0;
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_stop_transaction(
    const arm_tran_start_handle_t tran_handle,
    const arm_tran_status_t tran_status,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_update_transaction(
    const arm_tran_start_handle_t tran_handle,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_discard_transaction(
    const arm_tran_start_handle_t tran_handle,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_block_transaction(
    const arm_tran_start_handle_t tran_handle,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4,
    arm_tran_block_handle_t *block_handle)
{
	if (block_handle == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;

	/* Initialize the handle */
	*block_handle = 0;
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_unblock_transaction(
    const arm_tran_start_handle_t tran_handle,
    const arm_tran_block_handle_t block_handle,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	return ARM_SUCCESS;
}

/* thread support API functions */
ARM4_API_DYNAMIC(arm_error_t)
arm_bind_thread(
    const arm_tran_start_handle_t tran_handle,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	/* Process/thread level binding isn't currently supported */
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_unbind_thread(
    const arm_tran_start_handle_t tran_handle,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	/* Process/thread level binding isn't currently supported */
	return ARM_SUCCESS;
}

/* report transaction data API function */
ARM4_API_DYNAMIC(arm_error_t)
arm_report_transaction(
    const arm_app_start_handle_t app_handle,
    const arm_id_t *tran_id,
    const arm_tran_status_t tran_status,
    const arm_response_time_t response_time,
    const arm_stop_time_t stop_time,
    const arm_correlator_t *parent_correlator,
    const arm_correlator_t *current_correlator,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4)
{
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_generate_correlator(
    const arm_app_start_handle_t app_handle,
    const arm_id_t *tran_id,
    const arm_correlator_t *parent_correlator,
    const arm_int32_t flags,
    const arm_buffer4_t *buffer4,
    arm_correlator_t *current_correlator)
{
	if (current_correlator == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;

	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_get_correlator_length(
    const arm_correlator_t *correlator,
    arm_correlator_length_t *length)
{
	if (length == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;
	*length = 0;
	
	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_get_correlator_flags(
    const arm_correlator_t *correlator,
    const arm_int32_t corr_flag_num,
    arm_boolean_t *flag)
{
	if (flag == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;

	/* Correlators aren't currently supported, so the flag will never be set */
	*flag = ARM_FALSE;
	
	return ARM_SUCCESS;
}

/* miscellaneous API functions */
ARM4_API_DYNAMIC(arm_error_t)
arm_get_arrival_time(
    arm_arrival_time_t *opaque_time)
{
	if (opaque_time == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;
	*opaque_time = 0;

	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_get_error_message(
    const arm_charset_t charset,
    const arm_error_t code, 
    arm_message_buffer_t msg)
{
	strcpy (msg, "Unknown error");

	return ARM_SUCCESS;
}

ARM4_API_DYNAMIC(arm_error_t)
arm_is_charset_supported(
    const arm_charset_t charset,
    arm_boolean_t *supported)
{
	if (supported == NULL)
		return ARM_FAILURE_NULL_ARGUMENT;

	*supported = ARM_TRUE;
		
	return ARM_SUCCESS;
}

