/**********************************************************************
 * Copyright (c) 2005-2008 David Carter <dcarter@arm4.org> and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * David Carter - Initial API and implementation
 **********************************************************************/

#include "config.h"
#include <syslog.h>
#include <string.h>
#include <errno.h>

#include "Logger.h"

Logger * Logger::m_loggerPtr = NULL; // Singleton

Logger::Logger () :
    m_filePtr (NULL),
    m_namePtr ("")
{
}

Logger::~Logger ()
{
}

void
Logger::setFile (const std::string &name)
{
	if (name.length () > 0)
		m_filePtr = fopen (name.c_str (), "a");
}

void
Logger::openSyslog (void) const
{
    if (m_namePtr.length () > 0)
        openlog (m_namePtr.c_str (), LOG_PID, LOG_DAEMON);
    else
        openlog ("libarm4db", LOG_PID, LOG_DAEMON);
}

void
Logger::log (const char *format, ...) const
{
    va_list ap;

    va_start (ap, format);
    if (m_filePtr)
    {
        vfprintf (m_filePtr, format, ap);
        fflush (m_filePtr);
    }
    else
    {
        openSyslog ();
		vsyslog(LOG_ERR, format, ap);
    }
    va_end (ap);
}

void
Logger::error (const char *format, ...) const
{
    va_list ap;

    va_start (ap, format);
    if (m_filePtr)
    {
        vfprintf (m_filePtr, format, ap);
        fprintf (m_filePtr, ": %s\n", strerror(errno));
        fflush (m_filePtr);
    }
    else
    {
        openSyslog ();
        vsyslog(LOG_ERR, format, ap);
        syslog(LOG_ERR, strerror(errno));
    }
    va_end (ap);
}

void
Logger::exceptionError (const Arm4dbException &e) const
{
    if (m_filePtr)
    {
		fprintf (m_filePtr, "Exception: %s", e.getReasonC ());
        fflush (m_filePtr);
    }
    else
    {
        openSyslog ();
		syslog (LOG_ERR, "Exception: %s", e.getReasonC ());
    }
}

void
Logger::exceptionError (const Arm4dbException &e, const char *format, ...) const
{
    va_list ap;

    va_start (ap, format);
    if (m_filePtr)
    {
		fprintf (m_filePtr, "Exception: %s", e.getReasonC ());
        vfprintf (m_filePtr, format, ap);
        fflush (m_filePtr);
    }
    else
    {
        openSyslog ();
		syslog (LOG_ERR, "Exception: %s", e.getReasonC ());
        vsyslog (LOG_ERR, format, ap);
    }
    va_end (ap);
}

Logger &
Logger::getLogger (void)
{
    if (m_loggerPtr == NULL)
        m_loggerPtr = new Logger ();

    return *m_loggerPtr;
}

